#!/usr/bin/env python
##############################################################################
#
# (c) 2025 Sangjoon Lee.
# All rights reserved.
#
# File coded by: Sangjoon Lee, Anton Oliynyk, Emil Jaffal, Danila Shiryaev.
#
# See GitHub contributions for a more detailed list of contributors.
# https://github.com/bobleesj/structure-analyzer-featurizer/graphs/contributors
#
# See LICENSE.rst for license information.
#
##############################################################################
"""Python package to generate geometric features of interatomic
distances, atomic environment information, and coordination numbers from
a folder containing .cif files."""

# package version
from SAF.version import __version__  # noqa

# silence the pyflakes syntax checker
assert __version__ or True

# End of file
